(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 * faudra trouver un super entte pour mettre sur nos fichiers
 *
 *
 *)

(* Position : types position, direction et fonctions sur ces types *)

(* pour les directions, 
 *
 *               ^
 *    d_u (0,-1) | 
 * d_l (-1,0) <-   -> d_r (1,0)
 *               | d_d (0,1)
 *               V
 *)

type pos_t = int * int

(* obtention d'ensembles de positions *)

(* disque centr en (x,y) de rayon r *)
let disc (x,y) r = 
  let res = ref [(x,y)] in
    for i=1 to r do
      for j=1 to truncate(sqrt(float(r*r-i*i))) do
	res := (x+i,y+i)::(x+i,y-i)::(x-i,y+i)::(x-i,y-i) :: !res
      done;
      res := (x+i,y)::(x-i,y)::(x,y+i)::(x,y-i) :: !res
    done;
    !res

(* trait de (x1, y1)  (x2, y2) *)
let rec line (x1, y1) (x2, y2) = 
  let  dx = x2 - x1 and  dy = y2 - y1 in
  let round f = int_of_float (floor (f +. 0.5)) in
  let res = ref [] in
  if abs dy < abs dx then 
    (* plutt horizontal *)
    let f = float dy /. float (abs dx) in
    let s = if dx > 0 then 1 else -1 in
    let rx = ref x1 and fy = ref (float y1) in
      while !rx != x2  do
	res := (!rx, round !fy) :: !res;
	fy := !fy +. f;
	rx := !rx + s;
      done;
      (!rx, round !fy) :: !res
  else if dy = 0 then 
    (* un seul point *)
    [(x1, y1)]
  else
    (* plutt vertical *)
    let f = float dx /. float (abs dy) in
    let s = if dy > 0 then 1 else -1 in
    let ry = ref y1 and fx = ref (float x1) in
      while !ry != y2  do
	res := (round !fx, !ry) :: !res;
	fx := !fx +. f;
	ry := !ry + s;
      done;
      (round !fx, !ry) :: !res
      


(* rectangle (x1, y1) (x2, y2) *)
let rec rect (x1, y1) (x2, y2) = 
  if      x1>x2 then rect (x2, y1) (x1, y2)
  else if y1>y2 then rect (x1, y2) (x2, y1)
  else
    let res = ref [] in
      for i=x1 to x2 do
	for j=y1 to y2 do
	  res := (i,j) :: !res
	done;
      done;
      !res

(* translation d'un ensemble de positions *)
let translate (dx, dy) =
  List.map (fun (x,y) -> (x+dx,y+dy))

(* ... *)


(* type direction *)
type dir_t = int * int

(* les quatre directions *)
let d_d = (0,1)
let d_u = (0,-1)
let d_l = (-1,0)
let d_r = (1,0)

(* la direction nulle *)
let null_dir = (0,0)

(* dplacement selon une direction *)
let walk (px,py) (dx,dy) = 
  (px+dx, py+dy)

(* rotation  gauche *)
let left  (dx,dy) = (dy, -dx)

(* rotation  droite *)
let right (dx,dy) = (-dy, dx)

(* demi-tour *)
let back  (dx,dy) = (-dx, -dy)

(* la direction est-elle en x ou en y ? *)
let orient = function
  | (0,_) -> `Y
  |   _   -> `X


(* sortie cran... *)
let print_pos (px, py) = 
  Printf.printf "(%d, %d)" px py

let print_dir = function
  | d when d=d_u -> print_string "'up'" 
  | d when d=d_d -> print_string "'down'"
  | d when d=d_l -> print_string "'left'" 
  | d when d=d_r -> print_string "'right'"
  | d when d=null_dir -> print_string "'null'"
  | d            -> print_pos d

let print_char_dir = function
  | d when d=d_u -> print_char '^' 
  | d when d=d_d -> print_char 'v'
  | d when d=d_l -> print_char '<' 
  | d when d=d_r -> print_char '>'
  | _            -> print_char 'o'
