(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 * faudra trouver un super entte pour mettre sur nos fichiers
 *
 *
 *)

(* Position : types position, direction et fonctions sur ces types *)

(* pour les directions, 
 *
 *               ^
 *    d_u (0,-1) | 
 * d_l (-1,0) <-   -> d_r (1,0)
 *               | d_d (0,1)
 *               V
 *)


(* type position *)
type pos_t = int * int

(* obtention de diverses ensembles de positions *)
(* - disque centr en (x,y) de rayon r *)
val disc : pos_t -> int -> pos_t list

(* - trait de (x1, y1)  (x2, y2) *)
val line : pos_t -> pos_t -> pos_t list

(* - rectangle (x1, y1) (x2, y2) *)
val rect : pos_t -> pos_t -> pos_t list

(* - translation d'un ensemble de positions *)
val translate : pos_t -> pos_t list -> pos_t list


(* type direction *)
(* c'est joli qu'il soit abstrait, mais peut-tre chiant...  voir *)
type dir_t (* = int * int *)


(* fonctions d'affichage *)
val print_pos: pos_t -> unit
val print_dir: dir_t -> unit
val print_char_dir: dir_t -> unit


(* les quatre directions *)
val d_u : dir_t (* up    *)
val d_d : dir_t (* down  *)
val d_l : dir_t (* left  *)
val d_r : dir_t (* right *)

(* la direction nulle *)
val null_dir: dir_t

(* dplacement selon une direction *)
val walk : pos_t -> dir_t -> pos_t


(* rotation  gauche *)
val left  : dir_t -> dir_t
(* rotation  droite *)
val right : dir_t -> dir_t
(* demi-tour *)
val back  : dir_t -> dir_t


(* "sens de la direction" *)
val orient : dir_t -> [`X | `Y]
