(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 * faudra trouver un super entte pour mettre sur nos fichiers
 *
 *
 *)

(* Robot : dfinition de la classe abstraite 'robot' *)

open Events
open Walking
open Jumping
open Syntax
open Objet
open Position


class virtual ['kind, 'world, 'coerce] robot world pos = 
object (self)
  (* un robot est un objet... *)
  inherit ['kind, 'world, 'coerce] objet [`Listen; `React] world pos as parent
    
  (* un robot peut se dplacer... *)
  inherit walking

  (* il peut aussi sauter... *)
  inherit jumping as jumper

  (* programme principal du robot *)
  val prgm = Prgm.create()
  method prgm = prgm
		       
  (* objet de type `robot *)
  method kind: 'kind = `Robot

  (* rinitialisation du programme du robot *)
  method reset = prgm#reset

  (* on nous dclare un vnement, on transmet  notre programme *)
  method notify = prgm#declare

  (* vrai ssi le robot vient de se cogner contre qqc *)
  val mutable boom_state = false
  method boom_state = boom_state

  (* action du robot, lors d'un tour *)
  (* TO-COMPLETE... *)
  method react =
    let rec onBusy = function
      | `First  -> 
	  prgm#declare Busy;
	  n_react `Second
      | `Second -> 
	  print_string "BOOM\n"    ;
	  boom_state <- true
    and n_react n =
      match prgm#get with
	| Nop     -> ()
	| PreJump -> self#preJump
	| Rotate_Left  -> self#left
	| Rotate_Right -> self#right
	| UTurn        -> self#back
	| Jump       -> (try self#jump       with Cell.Busy -> onBusy n) 
	| Step       -> (try self#walk       with Cell.Busy -> onBusy n) 
	| Step_Left  -> (try self#walk_left  with Cell.Busy -> onBusy n)
	| Step_Right -> (try self#walk_right with Cell.Busy -> onBusy n)
    in
      boom_state <- false;
      jumper#react;
      n_react `First

  (* un robot ne peut pas tre travers*)
  method is_valid (obj: 'coerce) (dir: dir_t) = false

  (* affichage (mode texte) *)
  method print = self#w_print;
    
end
