(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 * faudra trouver un super entte pour mettre sur nos fichiers
 *
 *
 *)

(* Scanner : classe facilitant la cration d'objets 'simples',
 * dont le comportement est bas sur la dtection d'objets 
 * dans un certain rayon
 *)

open Objet
open Position

type 'obj act_t = [  
| `Generator of ('obj list -> Events.event list)
| `Listener 
| `Reactor of ('obj list -> unit)
]

type zone_t = [  
| `Fixed    of pos_t list
| `Variable of (pos_t -> pos_t list)
]

let rec get_generator = function
  | []                -> (fun _ -> [])
  | `Generator g :: _ -> g
  | _::q              -> get_generator q

let rec get_reactor = function
  | []                -> (fun _ -> ())
  | `Reactor r :: _   -> r
  | _::q              -> get_reactor q


class virtual ['kind, 'world, 'coerce] scanner 
  (zone: zone_t) 
  (actions: 'coerce act_t list) 
  world
  pos = 
object (self)
  inherit ['kind, 'world, 'coerce] objet 
   ((List.map (function | `Generator _ -> `Generate | `Reactor _ -> `React | `Listener -> `Listen)) actions) 
   world 
   pos 
   as parent

  method zone = match zone with
    | `Fixed z -> z
    | `Variable f -> f self#pos

  method scan: 'coerce list = Cell.scan (world#cell_list self#zone)

  val generator = get_generator actions 
  method generate = generator self#scan
  
  val reactor = get_reactor actions 
  method react = reactor self#scan
  
end

