/*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */
/***************************/
/* dmo openGL / Glut / Tk */
/***************************/

/* 
   portage ocaml de la librairie GLM

   cette librairie permet de lire/crire/dessiner des objets
   au format WaveFront (.obj)
*/


#include "glm.h"

#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/misc.h>
#include <caml/alloc.h>

#include <stdio.h>

#define Float_val(val) ((GLfloat) Double_val(val))
#define Val_float(dbl) (copy_double((double)(dbl)))
#define Model_val(val) ((GLMmodel *) Int_val(val))
#define Val_model(mdl) (Val_int((int)(mdl)))


// fonctions * -> unit
#define ML_0(cname) \
CAMLprim value ml_##cname (value unit) \
{ CAMLparam1(unit); glm##cname (); CAMLreturn(Val_unit); }

#define ML_1(cname, conv1) \
CAMLprim value ml_##cname (value arg1) \
{ CAMLparam1(arg1); glm##cname (conv1(arg1)); CAMLreturn(Val_unit); }

#define ML_2(cname, conv1, conv2) \
CAMLprim value ml_##cname (value arg1, value arg2) \
{ CAMLparam2(arg1, arg2); glm##cname (conv1(arg1), conv2(arg2)); CAMLreturn(Val_unit); }

#define ML_3(cname, conv1, conv2, conv3) \
CAMLprim value ml_##cname (value arg1, value arg2, value arg3) \
{ CAMLparam3(arg1, arg2, arg3); glm##cname (conv1(arg1), conv2(arg2), conv3(arg3)); CAMLreturn(Val_unit); }

// fonctions * -> 'a
#define ML_1_(cname, conv1, conv) \
CAMLprim value ml_##cname (value arg1) \
{ CAMLparam1(arg1); CAMLreturn(conv (glm##cname (conv1(arg1)))); }


GLuint option_val(value val) {
  static CAMLlocal5(none, flat, smooth, texture, color);
  static CAMLlocal1(material);

  int affect = 1;

  if (affect) {
    register_global_root(&none);
    none     = hash_variant("none");
    register_global_root(&flat);
    flat     = hash_variant("flat");
    register_global_root(&smooth);
    smooth   = hash_variant("smooth");
    register_global_root(&texture);
    texture  = hash_variant("texture");
    register_global_root(&color);
    color    = hash_variant("color");
    register_global_root(&material);
    material = hash_variant("material");
    affect = 0;
  }
       if (val==none)     return GLM_NONE;
  else if (val==flat)     return GLM_FLAT;
  else if (val==smooth)   return GLM_SMOOTH;
  else if (val==texture)  return GLM_TEXTURE;
  else if (val==color)    return GLM_COLOR;
  else if (val==material) return GLM_MATERIAL;
  return 0;
}

GLuint loption_val(value val) {
  if (val == Val_int(0)) 
    return 0;
  return option_val(Field(val, 0)) | loption_val(Field(val, 1));
}

value Val_3_float_array(GLfloat *tab) {
  CAMLparam0();
  CAMLlocal1(ret);
  ret = alloc_shr(3*sizeof(double)/sizeof(value), Double_array_tag);
  Store_double_field(ret, 0, (double) *tab++);
  Store_double_field(ret, 1, (double) *tab++);
  Store_double_field(ret, 2, (double) *tab++);
  CAMLreturn(ret);
}


ML_2 (Scale,             Model_val, Float_val)
ML_1 (ReverseWinding,    Model_val)
ML_1 (FacetNormals,      Model_val)
ML_2 (VertexNormals,     Model_val, Float_val)
ML_1 (LinearTexture,     Model_val)
ML_1 (SpheremapTexture,  Model_val)
ML_1 (Delete,            Model_val)
ML_3 (WriteOBJ,          Model_val, String_val, loption_val)
ML_2 (Draw,              Model_val, loption_val)
ML_2 (Weld,              Model_val, Float_val)

ML_1_(ReadOBJ,           String_val, Val_model)
ML_1_(Unitize,           Model_val,  Val_float)

CAMLprim value Dimensions(value model) {
  CAMLparam1(model);
  GLfloat t[] = {0., 0., 0.};
  glmDimensions(Model_val(model), t);
  CAMLreturn (Val_3_float_array(t));
}
