(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(*
 * the major part of this file has been written by Robert Robert Bridson
 * textures.ml: helper functions for handling OpenGL textures 
 *)

type texture = nativeint

exception ReadPPM
exception ReadBMP
exception ReadJPG

let setparam () =
  GlTex.parameter ~target:`texture_2d (`mag_filter `linear);
  GlTex.parameter ~target:`texture_2d (`min_filter `linear_mipmap_linear);
  GlTex.parameter ~target:`texture_2d (`wrap_s `repeat);
  GlTex.parameter ~target:`texture_2d (`wrap_t `repeat)

(* Attempt to load a 2d texture from the PPM file specified by argument,
 * auomatically building a full mipmap from it. Returns the id of a new
 * texture object containing it.
 *)
external loadPPM_: string -> texture = "loadPPMTexture"
let loadPPM s = match loadPPM_ s with
  | id when id=Nativeint.zero  -> raise ReadPPM
  | id -> id 

(* Attempt to load a 2d texture from the BMP file specified by argument,
 * auomatically building a full mipmap from it. Returns the id of a new
 * texture object containing it.
 *)
external loadBMP_: string -> texture = "loadBMPTexture"
let loadBMP s = match loadBMP_ s with
  | id when id=Nativeint.zero  -> raise ReadBMP
  | id -> id

(* Attempt to load a 2d texture from the JPG file specified by argument,
 * auomatically building a full mipmap from it. Returns the id of a new
 * texture object containing it.
 *)
external loadJPG_: string -> texture = "loadJPGTexture"
let loadJPG s = match loadJPG_ s with
  | id when id=Nativeint.zero  -> raise ReadJPG
  | id -> id

(* Bind the given texture object *)
external bind: texture -> unit = "bind2dTexture"

(* free the given texture object *)
external delete: texture -> unit = "deleteOnetexture"

