(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*** Lexer pour rogo ***)
{
  open Syntax;;
  open Parser;;
  exception Lexerror;;
}

rule token = parse

(** Tokens de base **)
  | ['0'-'9']['0'-'9']*       {INT(int_of_string(Lexing.lexeme lexbuf))}         (* nombres *)
  | "champignon"              {MUSHR}     (* un objet *)
  | '('                       {LPAR}
  | ')'                       {RPAR}
  | ";;"                      {SEMISEMI}
  | ';'                       {CMDSEP}
  | [ '\t' ' ' '\n' ] {token lexbuf}

(** Contrle du robot **)

(* Dplacements *)
  | "av" (""|"ance")          {STEP}         (* un pas en avant *)
  | "td"                      {ROT_R}         (* rotation horaire *)
  | "tg"                      {ROT_L}         (* rotation trigo *)
  | "dt"                      {UTURN}         (* demi-tour *)
  | "ag"                      {STEP_L}         (* va sur la case de gauche *)
  | "ad"                      {STEP_R}         (* va sur la case de droite *)
  | "re" (""|"cule")          {STEP_B}         (* un pas en arrire *)
      (*  dbattre : est-ce que le robot fait aussi un demi-tour ? *)
  | "sa" (""|"ute")           {JMP}         (* saute *)
(* Gestion de la marque (peinture ?) *)
  | "tr" (""|"ace")           {MARK}         (* met en marche le marqueur *)
  | "dt" | "detrace"          {UNMARK}         (* supprime le marqueur *)
(*  | "co" (""|"uleur")         {COL}         (* couleur du marqueur *) *)
(* Pathfinding *)
  | "vers"                    {STEPTO}       (* va vers un objet (1 pas) *)
  | "va"                      {GOTO}         (* se place  ct d'un objet (pathfinding)*)
(* Affichage *)
  | "dis"                     {DISP}        (* affiche un message *)
(* Actions *)
  | "pr" (""|"ends")          {TAKE}         (* prends un objet *)
  | "po" (""|"se")            {PUT}         (* pose un objet *)
  | "act" (""|"ive")          {ACT}        (* active (e.g. un bouton) *)
 

(** Flot d'execution **)

(* Interruption du flot *)
  | "ar" ( "" | "rete")       {BREAK}      (* sors d'une boucle avant sa terminaison *)
  | "stop"                    {STOP}       (* termine le programme du robot *)

(* For *)
  | "repete"                  {REP}
  | "fois"                    {TIMES}
  | "fin"                     {END}

(* Conditions *)

  | "si"                      {IF} 
  | "sinon"                   {ELSE}
  | "fais"                    {DO}
  | "tantque" | "tq"          {WHILE}
  | "quand"                   {WHEN}

(* Affectations *)

  | "soit"                    {LET}
  | "="                       {EQ}
  | ['a'-'z'] ['a'-'z' '0'-'9']*          
                              {VARNAME(Lexing.lexeme lexbuf)}
  | "change"                  {CHANGE}

(* Oprations *)
  | '+' {PLUS}
  | '-' {MINUS}
  | '*' {MULT}
  | '/' {DIV}

(* Conditions *)
  | "<"                       {LT}
  | ">"                       {GT}
  | "<="                      {LE}
  | ">="                      {GE}
  | "<>"                      {NEQ}

  | _ {raise Lexerror}
