(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 * faudra trouver un super entte pour mettre sur nos fichiers
 *
 *
 *)

(* Prgm : la classe qui dfinit le comportement d'un robot, 
 *  partir du code entr par l'lve
 *)

open Events
open Syntax


(********** type de la classe prgm **********)
class type prgm_t =
object

  (* ajout/modification de gestionnaires d'vnements *)
  method set_handler   : event_kind_t -> ast_t -> unit
  method addto_handler : event_kind_t -> ast_t -> unit
  method rem_handler   : event_kind_t -> unit

  (* obtention de gestionnaires d'vnements *)
  method get_handler   : event_kind_t -> ast_t
  method has_handler   : event_kind_t -> bool

  (* rinitialisation du programme *)
  method reset: unit

  (* reste-t-il quelque chose  faire ? *)
  method nop: bool

  (* dclaration d'un vnement au programme *)
  method declare: event -> unit

  (* obtention de la prochaine action *)
  method get: action_t

end


(********** objet du type prcdent **********)
class prgm : prgm_t = 
object (self)

  (* pile du flot d'excution *)
  val flow = Stack.create()
  method private pop    = Stack.pop flow
  method private push a = Stack.push a flow
			    
  (* rinitialisation : on vide la pile *)
  method reset = Stack.clear flow

  (* reste-t-il quelque chose  faire ? *)
  method nop = 
    try 
      ignore (Stack.top flow);
      false
    with 
      | Stack.Empty -> true
	  (* Stack.is_empty n'existe pas dans OCaml 3.04 
	   * (ou alors on peut tous passer en 3.06)
	   *)

  (* table des gestionnaires d'vnements *)
  val handlers = Hashtbl.create 5

  (* ajout/modification de gestionnaires d'vnements *)
  method set_handler   e f = 
    self#rem_handler e;
    Hashtbl.add  handlers e f
  method addto_handler e f = 
    let f' = try Seq(self#get_handler e, f) with Not_found -> f in
      self#set_handler e f'
  method rem_handler = Hashtbl.remove handlers

  (* obtention de gestionnaires d'vnements *)
  method get_handler = Hashtbl.find   handlers
  method has_handler = Hashtbl.mem    handlers


  (* dclaration d'un vnement au programme *)
  method declare e =
    let ke = kind_of e in
      (* TO-THINK-ABOUT *)
      if(not (self#has_handler ke)) then
	((* TODO : demander de dfinir le handler 
	   attention on dit juste  l'interface graphique 
	   qu'on dfinit un autre handler 
	   l'interface (ou autre) doit se rappeler de l'vnement
	   qui est en cours de df
	*))
      else    
	(* il faut alors empiler le gestionnaire correspondant *)
	self#push (self#get_handler ke)
 

  (* obtention de la prochaine action *)
  method get = 
    try
      (match split self#pop with
	  (* si split renvoie Nop, il n'y a vraiment plus rien  faire 
	   *  ce niveau : au remonte donc dans la pile pour y chercher 
	   * une action *)
	| Nop, _     -> self#get
	  (* si split renvoie une action, suivie de Nothing, il n'y a plus 
	   * rien  faire non plus  ce niveau : on renvoie l'action, et 
	   * on se laisse remonter dans la pile *)
	| a, Nothing -> a
	  (* dans les autres cas, on renvoie l'action, mais on rempile
	   * le rsidu *)
	| a, r       -> self#push r; a
      )
    with
      (* on renvoie Nop si on atteint le bout de la pile *)
      |	Stack.Empty -> Nop
		  
end


(********** cration d'un tel objet **********)
let create () = new prgm


