(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Syntax : dfinition de l'arbre syntaxique des expressions guidant les robots *)

open Events

(*  suivre... ce ne sont l que des ides... *)


(** Gestion des expressions arithmtiques **)

(* Expressions arithmtiques *)

type arithm_t = 
  | Int of int 
      (*  | Float of float *)
  | Name of string 
  | Plus of arithm_t*arithm_t
  | Minus of arithm_t*arithm_t
  | Mult of arithm_t*arithm_t
  | Div of arithm_t*arithm_t
  | Opp of arithm_t
      (* et plus pour avoir des fonctions sur les nombres :
       *  | Seq of ast_t ?
       *  | Funapp of fun_t * (arithm_t list) ?
       * ->  discuter
       *)
;;

(* Simplification des expressions arithmtiques :
 *   on effectue les oprations n'impliquant que des constantes
 *)

(*  val plus : arithm_t -> arithm_t -> arithm_t *)
let arithm_plus e1 e2 = match e1,e2 with
  | (Int a), (Int b) -> Int (a+b)
      (*  | (Int a), (Float b) -> Float ((float_of_int a) +. b)
       *  | (Float a), (Int b) -> Float ((float_of_int b) +. a)
       *  | (Float a), (Float b) -> Float (a +. b)
       *)
  | _,_ -> Plus(e1,e2)
;;

(*  val minus : arithm_t -> arithm_t -> arithm_t *)
let arithm_minus e1 e2 = match e1,e2 with
  | (Int a), (Int b) -> Int (a-b)
  | _,_ -> Minus(e1,e2)
;;

(*  val mult : arithm_t -> arithm_t -> arithm_t *)
let arithm_mult e1 e2 = match e1,e2 with
  | (Int a), (Int b) -> Int (a*b)
  | _,_ -> Mult(e1,e2)
;;

(*  val div : arithm_t -> arithm_t -> arithm_t *)
let arithm_div e1 e2 = match e1,e2 with
  | (Int a), (Int b) -> Int (a+b)
  | _,_ -> Div(e1,e2)
;;

(*  val opposite : arithm_t -> arithm_t *)
let arithm_opposite e1 = match e1 with
  | Int a -> Int( -a)
  | _ -> Opp(e1)
;;

(**  Conditions **)

type cond_t = 
  | Eq of arithm_t * arithm_t
  | Neq of arithm_t * arithm_t
  | Le of arithm_t * arithm_t
  | Ge of arithm_t * arithm_t
  | Lt of arithm_t * arithm_t
  | Gt of arithm_t * arithm_t
  | Not of cond_t
  | True 
  | False
  | And of cond_t * cond_t 
  | Or of cond_t * cond_t

(* actions 'relles' (marcher, tourner...) *)
type action_t =
  | Nop   
  | PreJump
  | Jump
  | Step
  | Step_Left
  | Step_Right
  | Step_Back
  | Rotate_Left
  | Rotate_Right
  | UTurn
  | Take
  | Put
  | Act
  | SetMark
  | UnsetMark
(* ... *)


(* programme : on rajoute le controle du flot *)
type ast_t = 
  | Nothing
  | Action  of action_t
  | Seq     of ast_t * ast_t
  | Repeat  of ast_t
  | RepeatN of int * ast_t
  | IfThenElse of cond_t * ast_t * ast_t
(* ... *)



(* 
 * split ast renvoie un couple form de la premire action de <ast>  effectuer 
 * et de l'expression qu'il faudra valuer ensuite
 * (par exemple, split (RepeatN (5, (Seq(Action Walk, Action Left)))) = a,e 
 *  o a = Walk 
 *  et e = Seq (Action Left, RepeatN (4, (Seq(Action Walk, Action Left))))
 * 
 * je trouve perso que c'est une jolie faon d'avancer dans l'arbre pas  pas...
 * mais c pas dit que a plaise  tt le monde (damien)
 *)
let rec split = function 
  | Nothing        -> Nop, Nothing
  | Action a       -> a, Nothing
  | Seq (e1,e2)    -> 
      (match split e1 with
	 | Nop, Nothing   -> split e2
	 | Nop, q         -> split (Seq(q, e2))
	 | a, Nothing     -> a, e2
	 | a, q           -> a, Seq(q, e2)
      )
  | Repeat e as re ->       
      (match split e with
	 | a, Nothing -> a, re
	 | a, q       -> a, Seq(q, re)
      )
  | RepeatN(0,_)   -> Nop, Nothing  
  | RepeatN(n,e)   ->       
      (match split e with
	 | a, Nothing -> a, RepeatN(n-1, e)
	 | a, q       -> a, Seq(q, RepeatN(n-1, e))
      )
 
