(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Main : le programme principal *)

open Gl_world;;
open World;;
open Syntax;;
open Position;;

class ['obj] glou_world = 
object(self)
  inherit ['obj] gl_world
  initializer
    cells <- Array.init 5 (fun i -> Array.init 5 (fun j -> Gl_cell.create (i,j)));
    ignore(Gl_mushroom.create  self (2,1));
    ignore(Gl_flower.create    self (1,3));
    ignore(Gl_blocker.create   self (1,3));
    ignore(Gl_bridge.create `X self (2,0));
(*    ignore(Gl_hedge.create  `Y self (0,2)); *)
    ignore(Gl_killer.create (`Fixed (disc (4,0) 1)) self (4,0));
    ignore(Gl_win.create    (`Fixed [4,4])          self (4,4));
end
;;


let interactive = ref false;;
Arg.parse
  ["-i", Arg.Set interactive, "\t mode interactif";] 
  (fun s -> raise (Arg.Bad s))
  "\nrigobot v5.1 - projet_mim2_rogo@ens-lyon.fr\n\nusage : rigobot [-i]"
;;

let lexbuf = Lexing.from_channel stdin;;
let get() = Language.get_ast lexbuf;;


let w = new glou_world in
let r = Gl_robot.create    w (0,4) in

  if not !interactive then (
    r#prgm#set_handler Events.K_Start (Seq(Seq(Action Step, Action Rotate_Right), Repeat(Action Step)));
    r#prgm#set_handler Events.K_Busy  (Action Step_Right); 
    (*
      r#prgm#set_handler Events.K_Start (Repeat(Action Step));
      r#prgm#set_handler Events.K_Busy  (Seq (Action PreJump, Action Jump)); 
    *)
    (*  r#prgm#set_handler Events.K_Busy  (Seq(Seq(Action Step_Right, Action Step_Right), Repeat(Action Step))); *)
    r#prgm#declare Events.Start;
  );
    
  w#print;
  flush stdout;
  if not !interactive then (
    print_string "press enter..."; flush stdout;
    ignore(read_line());
  );

  while true do
    if !interactive && r#prgm#nop then (
      print_string "instruction ? "; flush stdout;
      r#prgm#set_handler Events.K_Interactive (get());
      print_string "ok !"; print_newline();
      r#prgm#declare Events.Interactive;
    ) else (
      print_string "press enter..."; flush stdout;
      ignore(read_line());
    );
    (match w#next with
      | Won  -> print_string "WON !!!\n"; w#print; exit 0
      | Lost -> print_string "LOST !!!\n"; w#print; exit 0
      | _    -> print_string "----- world reacted ! -----\n\n");
    w#print;
  done

