(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Numbered : une classe  hriter, qui fournit un identifiant unique *)
(* voir l'exemple d'utilisation  la fin... *)


module type NULL     = sig end
module Null: NULL    = struct end 

module type NUMBERER = sig 
  class type numbered_t = object method id: int end
  class numbered: numbered_t 
end

module Create = functor (N: NULL) -> 
  (
    struct
      type t = int
      let counter = ref 0
      class type numbered_t = object
	method id: int 
      end
      class numbered = object
	val id = !counter
	method id = id
	initializer
	  incr counter
      end  
    end : NUMBERER
  )


(* 
   utilisation :
 
    module Id1 = Numbered.Create(Numbered.Null)
    module Id2 = Numbered.Create(Numbered.Null)

    class toto = object(self)
      inherit Id1.numbered
      method print = 
         Printf.printf "mon id est %d" self#id;
         print_newline()
    end

    class titi = object(self)
      inherit Id1.numbered
      method equals (o: Id1.numbered) = 
         self#id = o#id
    end

    class tutu = object
      inherit Id2.numbered
    end

    let o0 = new toto in
    let o1 = new toto in
    let o2 = new toto in
    let i  = new titi in
    let u = new tutu in
      o0#print;
      o1#print;
      o2#print;
      if (i#equals (o1:> Id1.numbered)) then failwith "c louche ;-(";
      u#id


 *)
