(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* ce fichier est un 'commentaire' :) *)

(* structure globale du code *)

class type ['objs] world_t = object 
  method set: 'objs cell_t array -> unit
  method cell: int -> 'objs cell_t 
end
and ['objs] cell_t = object 
  method add: 'objs -> unit
  method objets: 'objs list 
  method v_draw: unit
end 
and ['kind, 'world, 'coerce] objet_t = object 
  method coerce: 'coerce  
  method cell: 'coerce cell_t
  method world: 'world 
  method kind: 'kind 
end;;

class virtual ['objs] cell = 
object (self)
  val mutable ol:'objs list = [] 
  method coerce: 'objs cell_t = (self :> 'objs cell_t)
  method add o = ol <- o::ol 
  method objets = ol 
  method virtual v_draw: unit
end;;

exception Invalid;;
class ['objs] invalid_cell : ['objs] cell_t= 
object (self)
  method add o = raise Invalid
  method objets = raise Invalid
  method v_draw = ()
end;;

class ['objs] world =
object 
  val invalid_cell = new invalid_cell
  val mutable cells: 'objs cell_t array = [||] 
  method set a = cells <- a 
  method cell i = try cells.(i) with _ -> invalid_cell
end;;

class virtual ['world, 'coerce] objet w c = 
object (self)
  method world: 'world = w 
  method cell: 'coerce cell_t = c 
  initializer
    c#add self#coerce
end;;

class virtual ['kind, 'world, 'coerce] o1 w c =
object
  inherit ['world, 'coerce] objet w c
  method kind : 'kind =`a
  method a = 3.4
end;;

class virtual ['kind, 'world, 'coerce] o2 w c =
object
  inherit ['world, 'coerce] objet w c
  method kind : 'kind =`b
  method b = 3
end;;

(*
ce n'est plus possible car coerce est dsormais virtuelle...

let w = new world in
let c = new cell in
let _ = w#set [|c|] in
let a = new o1 w c in
let b = new o2 w c in 
  (List.hd (w#cell 0)#objets)#kind;;
*)







class type drawable = object 
  method draw: unit 
end;;

class type ['objs] g_world_t = object 
  inherit ['objs] world_t 
  inherit drawable 
end
and ['kind, 'world] g_objet_t = object 
  inherit ['kind, 'world, ('kind, 'world) g_objet_t] objet_t 
  inherit drawable 
end;;

class ['objs] g_cell = 
object (self)
  inherit ['objs] cell
  method draw = print_string "cell"; print_newline(); List.iter (fun o -> o#draw) ol
  method v_draw = self#draw
end;;

class ['objs] g_world =
object 
  inherit ['objs] world
  method draw = print_string "world"; print_newline(); Array.iter (fun o -> o#v_draw) cells
end;;

class virtual ['kind, 'world] g_objet = 
object (self)
  method coerce = (self :> ('kind, 'world) g_objet_t )
  method virtual cell: ('kind, 'world) g_objet_t cell_t
  method print = match self#kind with | `a -> "a" | `b -> "b" | _ -> "toto"
end;;

class ['kind, 'world] g_o1 w c =
object
  inherit ['kind, 'world] g_objet
  inherit ['kind, 'world, ('kind, 'world) g_objet_t] o1 w c
  method draw = print_string "o1 : a"; print_newline()
end;;

class ['kind, 'world] g_o2 w c =
object
  inherit ['kind, 'world] g_objet
  inherit ['kind, 'world, ('kind, 'world) g_objet_t] o2 w c
  method draw = print_string "o2 : b"; print_newline()
end;;


let g_w = new g_world in
let g_c = (new g_cell)#coerce in
let _ = g_w#set [|g_c|] in
let g_a = new g_o1 g_w g_c in
let g_b = new g_o2 g_w g_c in
  g_w#draw;
  (List.hd (g_w#cell 0)#objets)#draw;
  (List.hd (g_w#cell 0)#objets)#kind, g_b#print
