(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
open GlMap3d
open HeightField
open Tk
open Glut



let frame = ref 0.
let s= ref "FPS 000.000000"
let timebase = ref 0.
		 
(* projection orthogonal pour que ca s'affiche dans un plan ortho *)

let setOrtho togl=
  let w = float (Togl.width  togl) in
  let h = float (Togl.height togl) in
    GlMat.mode `projection;
    GlMat.push();
    GlMat.load_identity ();
    GluMat.ortho2d ~x:(0.,w) ~y:(0.,h) ;
    GlMat.scale ~x:1.0 ~y:(-1.0) ~z:1.0 ();
    GlMat.translate ~x:0.0 ~y:(-.h) ~z:0.0 ();
    GlMat.mode `modelview

let resetPro ()=
  GlMat.mode `projection;
  GlMat.pop();
  GlMat.mode `modelview

let displayFps togl=
  let h = float (Togl.height togl) in
    frame:= !frame +. 1.;
    let time =float (Glut.get ~gtype:ELAPSED_TIME ) in
      if(time -.(!timebase)> 1000.)then
	(let a=(!frame)*.1000.0/.(time-.(!timebase))in
	   Printf.printf "FPS %f\n" a;
	   flush stdout;
	   frame:=0.;
	   timebase:=time
	)

      
let onKeyPress touch z rotx roty dx dy view togl= match touch with
  | "Up" -> z:=!z+.3.
  | "Down" -> z:=!z-.3.
  | "Left" -> roty:=!roty+.1.
  | "Right" -> roty:=!roty-.1.
  | "a" -> rotx:=!rotx+.1.
  | "q" -> rotx:=!rotx-.1.
  | "u" -> dx:=!dx+1
  | "n" -> dx:=!dx-1
  | "h" -> dy:=!dy-1
  | "k" -> dy:=!dy+1
  | "e" -> view:=not (!view)
  | "p" -> displayFps togl
  | _ -> ()
	  
let main (glMapObj:GlMap3d.t)=
  let top = openTk () in
  let togl = Togl.create ~width:800 ~height:600 ~rgba:true ~depth:true ~double:true top in
  let z= ref 0. in
  let rotx= ref 0. in
  let roty= ref 0. in
  let dx = ref 37 in
  let dy = ref 37 in
  let view = ref true in
    Tk.bind top
      ~events:[`KeyPress]
      ~fields:[`KeySymString]
      ~action:(fun ev -> onKeyPress ev.Tk.ev_KeySymString z rotx roty dx dy view togl);
    glMapObj#init 2 0.;
    ignore(Glut.init Sys.argv);
    
    Wm.title_set top "Coucou";
    pack ~fill:`Both [togl];
    Gl.enable `depth_test;


    (*
      GlDraw.shade_model `smooth;
      GlLight.light_model (`local_viewer(1.0));

      Gl.enable `lighting;
      Gl.enable `light0;
      GlLight.light ~num:0 (`spot_direction(0.0,0.0,0.0));
      GlLight.material `both (`ambient((1.0,1.0,1.0,1.0)));
      Gl.enable `fog;
    *)

    Gl.enable `texture_2d;

    (* GlDraw.polygon_mode ~face:`both `line;  *)
    Togl.timer_func ~ms:1 ~cb:
      begin fun () ->
	Togl.render togl;
	displayFps togl
      end;

    Togl.display_func togl ~cb:
      begin fun () ->
	GlClear.clear [`color; `depth];
	GlMat.mode `projection;
	GlMat.load_identity ();
	GluMat.perspective ~fovy:45.0 ~aspect:1.0 ~z:(0.1,400.0);
	
	GlMat.translate3 (0.,0.,-110.+.(!z));	
	GlMat.rotate3 ~angle:((!rotx)) (1.,0.,0.);
	GlMat.rotate3 ~angle:((!roty)) (0.,1.,0.);
	let (dimx,dimy)= glMapObj#get_dim () in
	let (sdimx,sdimy)= glMapObj#get_squareDim () in
	  GlMat.translate3 (-.(sdimx*.(float_of_int dimx))/.2.,-.(sdimy*.(float_of_int dimy))/.2.,0.); 
	  
	  GlMat.mode `modelview;
	  
	  Gl.enable `depth_test;
	  
	  if (!view) then (
	    glMapObj#draw_all ();
	  )
	  else (
	    glMapObj#draw (!dx,!dy) (0.,20.);
	  );
  
	  Gl.disable `depth_test;

	  Togl.swap_buffers togl; 
	  

	  

      (*Gl.flush ()*)
	    
      end;
    Tk.mainLoop ()

let _ = Random.self_init ()
;;      

let _ = 
  let obj = GlMap3d.glMap3d_from_file "land.bmp" in  
  (*let h = HeightField.gen_height 101 101 in
    HeightField.put_relief h (HeightField.Hill(2.,0.3)) (25,25);
    HeightField.put_relief h (HeightField.Lack(5.,0.3)) (30,80);
    HeightField.put_relief h (HeightField.Plateau(5.,0.4,(40,40))) (100,30);
    HeightField.put_relief h (HeightField.Valley(5.,0.4,(40,40))) (100,100);
    let obj= new GlMap3d.glMap3d h 0 in *)
      begin
	main obj;
	if not !Sys.interactive then begin
	  exit 0;
	end;
      end;;

