(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
type vector3={x:float;y:float;z:float};;

let add (x,y,z) (x1,y1,z1)=
  (x+.x1,y+.y1,z+.z1)

let sous (x,y,z) (x1,y1,z1)=
  (x-.x1,y-.y1,z-.z1)

let dist (x,y,z) =
  sqrt ((x*.x)+.(y*.y)+.(z*.z))

let scale (x,y,z) a =
  ((x*.a),(y*.a),(z*.a))

let rY (x,y,z) a =
  let d=dist (x,y,z) in
  (d*.sin (a*.3.14/.180.0), y,d*.cos ((a)*.3.14/.180.0))


let rX (x,y,z) a =
  let d=dist (x,y,z) in
  (x, d*.sin (a*.3.14/.180.0),d*.cos ((a)*.3.14/.180.0))


let ortho (a,b,c) =
  (c,0.,-.a)


let print (x,y,z) =
  Printf.printf " (%f,%f%f)\n" x y z;
  flush stdout


class gl_camera =
  object(self)
    val mutable pos=(0.,1.,0.0)
    val mutable view=(2.,1.,0.)
    val mutable up=(0.,1.,0.)
    val mutable rotX=0.
    val mutable rotY=90.
    val mutable rotZ=0.
    val mutable maxx=0
    val mutable maxz=0

    val mutable changeView = false

    method getView =
      ()

    method rotatX a =
      rotX<-rotX +. a;
      view<- add (rX (sous view pos) rotX) pos 
      

    method rotatY a =
      rotY<-rotY +. a;
      view<- add (rY (sous view pos) rotY) pos ;
      

    method move b=
      let (x,y,z)=pos in
      let (bx,by,bz)=b in
      if(bx+.x <  (float maxx)) then
	if(bx+.x >0.) then
	   if(bz+.z <(float   maxz)) then
	     if(bz+.z >0.) then(
	       pos<-add pos b;
	       view<-add view b
	      )

    method moveForward a =
      let c= sous view pos in
      let d= dist c in
      self#move (scale  c (a/.d))

    method strafe  a =
      let c= sous view pos in
      let d= dist c in
      let o= ortho c in
      self#move (scale  o (a/.d))

    method moveup a=
      self#move (0.0,a,0.0)

    method setup a=
      let (px,py,pz)=pos in
      let (vx,vy,vz)=pos in
      pos<-(px,a,pz);
      view<-(vx,a+.(vy-.py),vz)

    method get_x=
      let (px,py,pz)=pos in
      px

    method get_y=
      let (px,py,pz)=pos in
      py
    method get_z=
      let (px,py,pz)=pos in
      pz
    method draw =
      GluMat.look_at 
        ~eye:   pos
        ~center:view
        ~up:    up
	
    method initmax (dx,dy)=
      maxx<-(dy);
      maxz<-(dx)
      
end
