(*
 *
 * Copyright (C) 2002 Aurelien Moreau, Antoni Gutsens Diaz, Benjamin Leveque,
 * 	 Damien Pous, David Roger, Lionel Vaux, Mathieu Martin, Nicolas Bernard,
 * 	 Samuel Thibault, Simon Capern, Thomas Gazagnaire
 * 
 * This file is part of Rigobot.
 * 
 * Rigobot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Ribogot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ribogot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)
(*
 * faudra trouver un super entte pour mettre sur nos fichiers
 *
 *
 *)

(* Gl_main : un monde reprsent dans le monde 3d *)

open Gl_robot
open Gl_world

open Glut
open Gl_camera







let fonts =  Glut.BITMAP_8_BY_13 ;;

let drawString x y s=
GlPix.raster_pos x y  ();
  for i=0 to ((String.length s)-1) do
    Glut.bitmapCharacter fonts  (int_of_char s.[i]);
  done
    ;;


 
let far =31.

class gl_main togl w=
object (self)


  val mutable frame = 0.
  val mutable s= "FPS 000.000000"
  val mutable timebase = 0.

  val mutable fog_density =
   1.0 -. Pervasives.exp (-5.5 *. 0.05 *. 0.05 *.far *. far)
  val mutable fog_param = (0.0,0.0,0.0,1.0)
  val mutable r1 = new gl_robot w   true
  val mutable world = w
  val mutable cam= new gl_camera


  method drawRep=

    GlDraw.begins `lines;

    GlDraw.color  (1.0,0.0,0.0);
    GlDraw.vertex3( 0.0, 0.0, 0.0 );
    GlDraw.vertex3( 5.0, 0.0, 0.0 );
    GlDraw.color  (0.0,1.0,0.0);
    GlDraw.vertex3( 0.0, 0.0, 0.0 );
    GlDraw.vertex3( 0.0, 0.0, 5.0 );
    GlDraw.color  (0.0,0.0,1.0);
    GlDraw.vertex3( 0.0, 0.0, 0.0 );
    GlDraw.vertex3( 0.0, 5.0, 0.0 );

    GlDraw.ends ()


  method onDisplay ()=
    let h=(world#glMapObj#get_z (cam#get_z,cam#get_x)) in
    cam#moveup ((1.0+. h)-.cam#get_y);
    r1#set_y ((world#glMapObj#get_z (r1#get_z,r1#get_x))-.1.0);

    GlClear.clear [`color; `depth];
   GlMat.mode `modelview;
    GlMat.load_identity ();
   let (a,b,c,d)= fog_param in
GlClear.color (a,b,c);
    cam#draw;
    self#drawRep;
 
    w#gl_draw;
  
    r1#gl_draw;
    self#displayFps;

    self#light();
    Togl.swap_buffers togl
  

  method setOrtho =
    let w = float (Togl.width  togl) in
    let h = float (Togl.height togl) in
    GlMat.mode `projection;
    GlMat.push();
    GlMat.load_identity ();
    GluMat.ortho2d ~x:(0.,w) ~y:(0.,h) ;
    GlMat.scale ~x:1.0 ~y:(-1.0) ~z:1.0 ();
    GlMat.translate ~x:0.0 ~y:(-.h) ~z:0.0 ();
    GlMat.mode `modelview

  method resetPro =
    GlMat.mode `projection;
    GlMat.pop();
    GlMat.mode `modelview

  method displayFps =
    let h = float (Togl.height togl) in
    frame<- frame +. 1.;
    let time =float (Glut.get ~gtype:ELAPSED_TIME ) in
    if(time -.timebase> 1000.)then
      (let a=frame*.1000.0/.(time-.timebase)in
      s<-Printf.sprintf  "FPS %f\n" a;
      frame<-0.;
      timebase<-time
      );
    self#setOrtho ;
    GlMat.push();
    GlMat.load_identity ();
    Gl.disable `lighting;
    GlDraw.color  (1.0,1.0,1.0);  
    drawString 2.0 (h-.300.0) s;
    Gl.enable `lighting;
    GlMat.pop();
    self#resetPro

  method onReshape ()= 
    let w = float (Togl.width  togl) in
    let h = float (Togl.height togl) in
    GlMat.mode `projection;
    GlMat.load_identity ();
    GlDraw.viewport 0 0 (int_of_float w) (int_of_float h);
    GluMat.perspective ~fovy:50.0 ~aspect:(w/.h) ~z:(0.1, far);

  method onKeyPress a= match a with
    | "Up"     -> cam#moveForward 1.0 ; 
	Togl.render togl
    | "Down"   -> cam#moveForward (-1.0) ;
	Togl.render togl
    | "Left"   -> cam#rotatY (2.0) ;
	Togl.render togl
    | "Right"  -> cam#rotatY (-2.0) ;
	Togl.render togl
    | "Next"  -> cam#moveup (-1.0) ;
	Togl.render togl
    | "Prior"  -> cam#moveup (1.0) ;
	Togl.render togl
    | "Home"  -> cam#strafe (-1.0) ;
	Togl.render togl
    | "End"  -> cam#strafe (1.0) ;
	Togl.render togl
    | "a"  -> r1#avance ;
	Togl.render togl
    | "z"  -> r1#recule ;
	Togl.render togl
    | s -> print_string s; print_newline()

  method light ()=

    GlLight.light ~num:0 (`position (0.,0.,0.,0.)) ;     
    GlMat.push();
    GlMat.translate3 (0.,5.,0.);
    Glut.solidSphere ~radius:0.1 ~slices:20 ~stacks:10;
    GlMat.pop(); 
    GlLight.light ~num:1 (`position (0.,5.,0.,0.)) ;     
    GlMat.push();
    GlMat.translate3 (0.,-5.,0.);
    Glut.solidSphere ~radius:0.1 ~slices:20 ~stacks:10;
    GlMat.pop();
    GlLight.light ~num:2 (`position (0.,-5.,0.,0.))   
       
  method init() =
 
    Gl.enable `fog;
    let (a,b,c,d)=fog_param in
    GlClear.color  (a,b,c);
(*    Gl.enable `cull_face;*)
    Gl.enable `depth_test;
(*    Gl.enable `dither;*)
    GlMisc.hint `perspective_correction `nicest;
   Gl.enable `blend;
    GlFunc.blend_func ~src:`src_alpha ~dst:`one_minus_src_alpha;
    Gl.enable `color_material;

    (* gestion des lumires *)
    GlLight.light_model (`two_side true);
    GlLight.light ~num:0 (`diffuse (1., 1., 1., 1.)) ; 
    GlLight.light ~num:1 (`diffuse (1., 1., 1., 1.)) ;
    GlLight.light ~num:2 (`diffuse (1., 1., 1., 1.)) ; 

    Gl.enable `lighting;
    Gl.enable `light1;  
    Gl.enable `light2;

   GlMat.mode `modelview;
 (*   GlLight.fog (`start  2.);
    GlLight.fog (`End  30.0);*)
   GlLight.fog (`mode  `linear);
    GlLight.fog (`density  (fog_density/.10.));
    GlLight.fog (`color fog_param);
    Gl.enable `fog

    
      

 initializer
   self#init();
   cam#initmax (world#glMapObj#get_dim()) ;
   let (a,b,c,d)= fog_param in
   fog_param<-(0.16+.0.05*.0.57,
	       0.31+.0.05*.0.45,
	       0.74+.0.05*.0.22,1.0)
	       
   
 
end
